<?php


/**
 * @file
 * Allow ec_donate_get_all('checkout') to hook into ec_donate
 */

function ec_donate_checkout_form(&$form, &$form_state) {
  if ($form_state['txn']->type != "ec_cart") {
    return;
  }

  // disable page caching
  global $conf;
 	$conf['cache']=CACHE_DISABLED; 
  
  $txn =& $form_state['txn'];
  // verify that this is allowed to happen
  if ( variable_get('ec_donate_oncheckout_enable', 0) == 0 ) {
    return;
  }

  // get all donations that can appear on checkout
  $nodes = _ec_get_oncheckout_donations($txn);
  if (count($nodes) == 0 ) {
    return;
  }
  $form['ec_donate'] = array();
  $form['ec_donate']['#theme'] = 'ec_donate_oncheckout';
  $form['ec_donate']['title'] = array(
    '#title' => t('Donation'),
  );
  $form['ec_donate']['amount_text'] = array(
    '#value' => t('Amount (in !currency)', array('!currency' => variable_get('ec_default_currency', 'your currency'))),
  );
  $form['ec_donate']['oncheckout_text'] = array(
    '#value' => variable_get('ec_donate_oncheckout_text', ''),
  );

  $form['ec_donate']['oncheckout_donations'] = array(
      '#tree' => TRUE,
  );
  foreach ($nodes as $node) {
    $form['ec_donate']['oncheckout_donations'][$node->nid] = array();
    $form['ec_donate']['oncheckout_donations'][$node->nid]['#theme'] = 'ec_donate_oncheckout_user_entered_price_element';
    $form['ec_donate']['oncheckout_donations'][$node->nid]['body'] = array(
      '#value' => $node->body,
    );
    $form['ec_donate']['oncheckout_donations'][$node->nid]['user_entered_price'] = array(
      '#type' => 'textfield',
      '#size' => 10,
    );
  }
}

function ec_donate_oncheckout_donation(&$form, &$form_state) {
}

function ec_donate_checkout_validate(&$form, &$form_state) {
  $txn = & $form_state['txn'];
    if ($form_state['values']['form_id'] == 'ec_checkout_form' && isset($form_state['values']['donations'])) {
    foreach ($form_state['values']['donations'] as $nid => $item) {
      // don't validate unless we have a value
      if (!empty($item['user_entered_price']) ) {
        $valid_amount = ec_donate_validate_donation_amount($item['user_entered_price']);
        if ($item['user_entered_price'] != $valid_amount ) {
          form_set_error('user_entered_price',
              t('Amount must be greater than or equal to @valid_amount', array('@valid_amount' => $valid_amount))
          );
        }
      }
    }
  }
}

function ec_donate_checkout_update(&$form, &$form_state) {
  $txn = & $form_state['txn'];
  if ($txn->type == "ec_cart" && isset($form_state['values']['donations'])) {
    foreach ( $form_state['values']['donations'] as $nid => $data ) {
      if (!empty($data['user_entered_price']) ) {
        // ensure that the cart has the item - just in case the user goes back to the cart
        ec_cart_update_item($nid, 1, $data);
      }
    }
    // ensure that the txn has the latest cart contents
    $txn->items = ec_cart_current();
    // ensure that the price for each donation is what the user entered
    foreach ( $txn->items as $item) {
      if ( isset($item->product_is_donation) && $item->product_is_donation ) {
        $item->price = $item->user_entered_price;
      }
    }
  }
}

/**
 * retrieve fully built nodes
 */
function _ec_get_oncheckout_donations(&$txn) {
  $nodes = array();
  $sql = 'SELECT node.nid
            FROM {ec_product}
            JOIN {node}
              ON node.nid = ec_product.nid
            WHERE display_on_checkout=1
              AND status=1
          ';
  $result = db_query($sql);
  while ($answer = db_fetch_object($result)) {
    // ensure that this donation is not already in the cart
    if ( !isset($txn->items[$answer->nid])) {
      $nodes[] = node_load($answer->nid);
    }
  }
  return $nodes;
}

/**
 * 
 * @param type $form
 * @param type $form_state 
 */
function ec_donate_alter_address(&$form, &$form_state) {
  $txn = & $form_state['txn'];
  $type = $form_state['clicked_button']['#parents'][1];
  $txn->{$type . '_address'} = $form_state['values']['address'][$type]['buttons']['select_address'];
  $txn->address['billing'] = ec_customer_get_addresses($txn->customer, $txn->{$type . '_address'});
}


/**
 * AJAX to allow the customer to change address.
 */
function ec_donate_update_address_js($type = NULL) {
  ctools_include('object-cache');

  if (empty($type) || 
    !($cached_form = form_get_cache($_POST['form_build_id'], $cached_form_state)) || 
    !($txn = ctools_object_cache_get('transaction', 'ec_donate'))) {
    form_set_error('form_token', t('Validation error, please try again. If this error persists, please contact the site administrator.'));
    $output = theme('status_messages');
    print drupal_to_js(array('status' => TRUE, 'data' => $output));
    exit();
  }

  $form_state = array('values' => $_POST);
  $form_state['txn'] = & $txn;

  $txn->{$type . '_address'} = $form_state['values']['address'][$type]['buttons']['select_address'];

  $form = array();

  if ($type == 'billing') {
    $address = ec_customer_get_addresses($txn->customer, $form_state['txn']->billing_address);
  }
  else {
    $address = ec_customer_get_addresses($txn->customer, $form_state['txn']->shipping_address);
  }
  
  module_load_include('inc', 'ec_customer','ec_customer.checkout');
  ec_customer_checkout_form($form, $form_state);

  $cached_form['ec_customer']['address'][$type] = $form['ec_customer']['address'][$type];

  $new_form = array('ec_customer' => array('address' => array('#tree' => TRUE)));
  $new_form['ec_customer']['address'][$type] = $form['ec_customer']['address'][$type];

  $new_form['ec_customer']['address'][$type]['display_address'] = array('#value' => theme('formatted_address', $address));

  $form = $new_form;

  form_set_cache($_POST['form_build_id'], $cached_form, $cached_form_state);

  // Render the form for output.
  $form += array(
    '#post' => $_POST,
    '#programmed' => FALSE,
    '#tree' => FALSE,
    '#parents' => array(),
  );

  unset($form['ec_customer']['address'][$type]['#prefix']);
  unset($form['ec_customer']['address'][$type]['#suffix']);
  if ($type == 'billing') {
    unset($form['ec_customer']['address'][$type]['use_for_shipping']);
  }

  drupal_alter('form', $form, array(), 'ec_customer_checkout_form');
  $form_state = array('submitted' => FALSE);
  $form = form_builder('ec_donate_donate_form', $form, $form_state);
  $output = theme('status_messages') . drupal_render($form);

  ctools_object_cache_set('transaction', 'ec_donate', $txn);
  // We send the updated file attachments form.
  // Don't call drupal_json(). ahah.js uses an iframe and
  // the header output by drupal_json() causes problems in some browsers.
  print drupal_to_js(array('status' => TRUE, 'data' => $output));
  exit;
}